package de.jottyfan.bico.modules.lesson;

import static de.jottyfan.bico.db.public_.Tables.T_LESSON;
import static de.jottyfan.bico.db.public_.Tables.T_PERSON;
import static de.jottyfan.bico.db.public_.Tables.T_SLOT;

import java.time.LocalDate;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.DeleteConditionStep;
import org.jooq.InsertResultStep;
import org.jooq.Record1;
import org.jooq.SelectConditionStep;
import org.jooq.SelectWhereStep;
import org.jooq.UpdateConditionStep;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.bico.db.public_.tables.records.TLessonRecord;
import de.jottyfan.bico.db.public_.tables.records.TPersonRecord;

/**
 *
 * @author jotty
 *
 */
@Repository
public class LessonRepository {
	private final static Logger LOGGER = LogManager.getLogger(LessonRepository.class);

	@Autowired
	private DSLContext jooq;

	public TLessonRecord getLesson(Integer slotId, Boolean createIfNecessary) {
		SelectConditionStep<TLessonRecord> sql = jooq
		// @formatter:off
			.selectFrom(T_LESSON)
			.where(T_LESSON.FK_SLOT.eq(slotId));
		// @formatter:on
		LOGGER.trace(sql);
		TLessonRecord r = sql.fetchOne();
		if (r == null && createIfNecessary) {
			InsertResultStep<TLessonRecord> sql2 = jooq
			// @formatter:off
				.insertInto(T_LESSON,
						        T_LESSON.FK_SLOT)
				.values(slotId)
				.returning(T_LESSON.PK_LESSON);
			// @formatter:on
			LOGGER.trace(sql);
			Integer pkLesson = sql2.fetchOne(T_LESSON.PK_LESSON);
			r = new TLessonRecord();
			r.setPkLesson(pkLesson);
			r.setFkSlot(slotId);
		}
		return r;
	}

	public List<TPersonRecord> getPersons() {
		SelectWhereStep<TPersonRecord> sql = jooq.selectFrom(T_PERSON);
		LOGGER.trace(sql);
		return sql.fetchStream().toList();
	}

	public LocalDate getSlotDay(Integer slotId) {
		SelectConditionStep<Record1<LocalDate>> sql = jooq
		// @formatter:off
			.select(T_SLOT.SLOT_DAY)
			.from(T_SLOT)
			.where(T_SLOT.PK_SLOT.eq(slotId));
		// @formatter:on
		LOGGER.trace(sql);
		return sql.fetchOne(T_SLOT.SLOT_DAY);
	}

	public void updateLesson(TLessonRecord bean) {
		UpdateConditionStep<TLessonRecord> sql = jooq
		// @formatter:off
			.update(T_LESSON)
			.set(T_LESSON.NOTES, bean.getNotes())
			.set(T_LESSON.FK_PERSON, bean.getFkPerson())
			.where(T_LESSON.PK_LESSON.eq(bean.getPkLesson()));
		// @formatter:on
		LOGGER.trace(sql);
		sql.execute();
	}

	public void removeLesson(Integer lessonId) {
		DeleteConditionStep<TLessonRecord> sql = jooq
		// @formatter:off
			.deleteFrom(T_LESSON)
			.where(T_LESSON.PK_LESSON.eq(lessonId));
		// @formatter:on
		LOGGER.trace(sql);
		sql.execute();
	}
}
